// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file RefinementManager1d.h */

#ifndef AMDIS_REFINEMENT_MANAGER_1D_H
#define AMDIS_REFINEMENT_MANAGER_1D_H

namespace AMDiS {

  /** \ingroup Adaption
   * \brief
   * Implements a RefinementManager for 1-dimensional meshes.
   */
  class RefinementManager1d : public RefinementManager
  {
  public:
    /// Calls base class constructor.
    RefinementManager1d()
      : RefinementManager()
    {}

    /// destructor 
    virtual ~RefinementManager1d() {}

    /// Implements RefinementManager::refineMesh.
    Flag refineMesh(Mesh *aMesh);

    /// Implements RefinementManager::setNewCoords
    void setNewCoords(int macroEl = -1);

  protected:
    /// Used by refineMesh while mesh traversal
    void recursiveRefineFunction(ElInfo* el_info);

    /// Used by \ref setNewCoords
    void newCoordsFct(ElInfo *el_info);
  };

}

#endif // AMDIS_REFINEMENT_MANAGER_1D_H
