// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file BddcMlSolver.h */

#ifndef AMDIS_BDDC_ML_SOLVER_H
#define AMDIS_BDDC_ML_SOLVER_H

#include "AMDiS_fwd.h"
#include "parallel/PetscSolver.h"

namespace AMDiS {

  using namespace std;

#ifdef HAVE_BDDC_ML

  class BddcMlSolver : public PetscSolver
  {
  public:
    /// Creator class
    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}

      /// Returns a new PetscSolver object.
      OEMSolver* create() 
      { 
	return new BddcMlSolver(this->name); 
      }
    };
    
    BddcMlSolver(string name)
      : PetscSolver(name),
	rhsVec(NULL),
	mat(NULL)
    {}

    void fillPetscMatrix(Matrix<DOFMatrix*> *mat);

    void fillPetscRhs(SystemVector *vec);

    void solvePetscMatrix(SystemVector &vec, AdaptInfo *adaptInfo);    

    void destroyMatrixData();

    void destroyVectorData();

  protected:
    void addDofMatrix(DOFMatrix* mat, 
		      vector<int>& i_sparse, 
		      vector<int>& j_sparse,
		      vector<double>& a_sparse,
		      int nComponents,
		      int ithRowComponent,
		      int ithColComponent);
  protected:
    SystemVector *rhsVec;

    Matrix<DOFMatrix*> *mat;
  };

#endif

}

#endif

