#include "AMDiS.h"
#include "ExtendedProblemStat.h"
#include "MeshFunction_Level.h"
#include "Refinement.h"
#include "SignedDistFunctors.h"

#include "boost/date_time/posix_time/posix_time.hpp"

using namespace AMDiS;
using namespace boost::posix_time;

int main(int argc, char** argv)
{ FUNCNAME("main");

  AMDiS::init(argc, argv);
  ptime start_time = microsec_clock::local_time();

  // initialize the problem
  // ======================
  ExtendedProblemStat prob("prob");
  prob.initialize(INIT_ALL);
  
  // refine the mesh
  // ===============
  double radius = 0.3;
  WorldVector<double> center; center.set(0.5);
  Parameters::get("refinement->radius", radius);
  Parameters::get("refinement->center", center);
  
  SignedDistRefinement refFunction("refinement", prob.getMesh());
  RefinementLevelCoords2 refinement(prob.getFeSpace(), &refFunction,
      new Circle(radius, center));

  // initial refinement
  // ==================
  refinement.refine(10);

  // Adapt-Infos
  AdaptInfo adaptInfo("adapt", prob.getNumComponents());  
  prob.writeFiles(adaptInfo, false);
  
  time_duration td = microsec_clock::local_time()-start_time;
  MSG("elapsed time = %f sec\n", td.total_milliseconds()/1000.0);

  AMDiS::finalize();

  return 1;
};
